/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Transfer Agents: AgentState Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: AgentState.inl,v 1.3 2006/08/23 14:31:42 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_3_0_2_2 $
 *
 */
#include "glite/data/transfer/agent/dao/AgentDAO.h"
#include "log4cpp/Category.hh"

#include <boost/scoped_ptr.hpp>
#include <unistd.h>

/*
 * Constants
 */
namespace {
const char * const AGENT_STATE_NAME         = "transfer-agent-state";
const unsigned int HEARTBEATS_FOR_OBSOLETE  = 2;
}

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
    
/*
 * AgentState
 *
 * Constructor
 */
template <class F>
AgentState<F>::AgentState(agents::dao::DAOContext& ctx,
                          const std::string& n,
                          const std::string& t,
                          const std::string& c,
                          unsigned int heartbeat_interval): 
    m_logger(log4cpp::Category::getInstance(AGENT_STATE_NAME)),
    m_ctx(ctx),
    m_name(n),
    m_type(t),
    m_contact(c),
    m_heartbeatInterval(heartbeat_interval){
}
                                                                                                                          
/*
 * ~AgentState
 *
 * Destructor
 */
template <class F>
AgentState<F>::~AgentState(){
}

/*
 * set
 *
 * Set Agent State
 */
template <class F>
void AgentState<F>::set(model::Agent::State s) /*throw(DAOException,LogicError)*/{
    
    // Get version
    char * version = GLITE_TRANSFER_AGENT_VERSION;
    
    // Get hostname
    char host_name[256];
    gethostname(host_name, 256);
    // Get current Time
    time_t current;
    time(&current);
    
    // Create an model::Agent instance
    boost::scoped_ptr<model::Agent> agent(new model::Agent(
        m_type,                     // agent_type
        s,                          // agent_state
        (0 != version)?version:"",  // agent_version
        host_name,                  // agent_host
        m_contact,                  // contact
        current));                  // last_active
    
    // Create DAO Context
    try{
        // Start Transaction
        m_ctx.start();
        
        // Create a AgentDAO instance
        boost::scoped_ptr<dao::AgentDAO> agentDao(
            F::instance().createAgentDAO(m_name,m_ctx));
    
        // Insert the Instance
        agentDao->create(*agent);

        // Commit Transaction
        m_ctx.commit();
    } catch(const agents::dao::DAOException& exc){
        m_logger.log(log4cpp::Priority::ERROR,"Can't Set Agent State. Reason is %s",exc.what());
        m_ctx.rollback();
        throw;
    } catch(const agents::LogicError& exc){
        m_logger.log(log4cpp::Priority::ERROR,"Can't Set Agent State (LogicError). Reason is %s",exc.what());
        m_ctx.rollback();
        throw;
    }
}

/*
 * checkAgent
 *
 * Check if an Agent is already running
 */
template<class F>
bool AgentState<F>::check() /*throw(DAOException,LogicError)*/ {
    
    bool result = false;
    
    if(0 == m_heartbeatInterval){
        // Heartbeat is disabled: Return always false
        return result;
    }
    
    // Create DAO Context
    try{
        // Create a AgentDAO instance
        boost::scoped_ptr<dao::AgentDAO> agentDao(
            F::instance().createAgentDAO(m_name,m_ctx));
    
        try{
            // Try to get the Agent Instance
            boost::scoped_ptr<model::Agent> agent(agentDao->get());

            // Check instance status
            if(model::Agent::S_STOPPED == agent->state()){
                // No Agent Running
                result = false;
            } else {
                // Check Host Name
                char host_name[256];
                gethostname(host_name, 256);
                if(agent->hostName() == host_name){
                    // Can't have two agents on the same box. 
                    // Probably the agent crashed
                    m_logger.log(log4cpp::Priority::WARN,"An instance of the Agent <%s:%s> found on that host. "\
                        "Probably the agent crashed",m_type.c_str(),m_name.c_str());
                    result = false;
                } else {
                    // Found an instance of the agent on another host:
                    // Check if it's valid
                    m_logger.log(log4cpp::Priority::WARN,"An instance of the Agent <%s:%s> found on host %s",
                        m_type.c_str(),m_name.c_str(),agent->hostName().c_str());
                    
                    // Get current Time
                    time_t current;
                    time(&current);
                    
                    // Compare current time with last update time
                    time_t last_active = agent->lastActive();
                    unsigned int obsolete_interval = m_heartbeatInterval * HEARTBEATS_FOR_OBSOLETE;
                    if(current > (last_active + obsolete_interval)){
                        m_logger.log(log4cpp::Priority::WARN,"The existing agent instance have not been updated for more than %d seconds. "\
                            "Probably the agent crashed",obsolete_interval);
                        result = false;
                    } else {
                        m_logger.log(log4cpp::Priority::WARN,"An existing Agent <%s:%s> is running on host %s",
                            m_type.c_str(),m_name.c_str(),agent->hostName().c_str());
                        
                        // Sleep for a while and check if the agent is still running
                        m_logger.log(log4cpp::Priority::INFO,"Waiting for %d seconds to check if the existing instance is still running...",
                            m_heartbeatInterval + 1);
                        sleep(m_heartbeatInterval + 1);
                        m_logger.log(log4cpp::Priority::DEBUG,"Check if agent has been updated");
                        
                        // Try to get the Agent Instance
                        agent.reset(agentDao->get());
                        if(last_active == agent->lastActive()){
                            // LastActive has not been Updated
                            m_logger.log(log4cpp::Priority::WARN,"The existing Agent <%s:%s> entry has not been updated in last %d seconds. "\
                                "Probably it refers to an obsolete agent instance that crashed",m_type.c_str(),m_name.c_str(),m_heartbeatInterval);
                            result = false;
                        } else {
                            m_logger.log(log4cpp::Priority::ERROR,"An Agent for <%s:%s> is already running",m_type.c_str(), m_name.c_str());
                            result = true;
                        }
                    }
                }
            }
        } catch(const agents::dao::DAOException& exc){
            // No Agent Running
            result = false;
        }
    } catch(const agents::dao::DAOException& exc){
        m_logger.log(log4cpp::Priority::ERROR,"Can't check Agent state: %s",exc.what());
        throw;
    } catch(const agents::LogicError& exc){
        m_logger.log(log4cpp::Priority::ERROR,"Can't check Agent state (LogicError): %s",exc.what());
        throw;
    }
    return result;
}

/*
 * Update
 *
 * Update Agent State
 */
template <class F>
void AgentState<F>::update(model::Agent::State s) /*throw(DAOException,LogicError)*/{
    
    // Get current Time
    time_t current;
    time(&current);

    // Create DAO Context
    try{
        // Create a AgentDAO instance
        boost::scoped_ptr<dao::AgentDAO> agentDao(
            F::instance().createAgentDAO(m_name,m_ctx));
    
        // Try to get the Agent Instance
        boost::scoped_ptr<model::Agent> agent(agentDao->get());
        
        // Update Fields
        agent->state = s;
        agent->lastActive = current;
        
        // Start Transaction
        m_ctx.start();
        
        // Insert the Instance
        agentDao->update(*agent);

        // Commit Transaction
        m_ctx.commit();
    } catch(const agents::dao::DAOException& exc){
        m_logger.log(log4cpp::Priority::ERROR,"Can't Update Agent State. Reason is %s",exc.what());
        m_ctx.rollback();
        throw;
    } catch(const agents::LogicError& exc){
        m_logger.log(log4cpp::Priority::ERROR,"Can't Update Agent State (LogicError). Reason is %s",exc.what());
        m_ctx.rollback();
        throw;
    }
}

} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite
