/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Agents Common: SDCache class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ActiveTxCache.h,v 1.3 2007/02/21 22:31:51 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_3_0_2_2 $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTIONS_CHANNEL_ACTIVETXCACHE_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTIONS_CHANNEL_ACTIVETXCACHE_H_

#include "glite/data/transfer/agent/model/Job.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Transfer.h"

#include <vector>
#include <string>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/utility.hpp>

namespace log4cpp{
    class Category;
}

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace channel {

/**
 * Forward declaration of SDCache Implementation
 */
class ActiveTxCacheImpl;
    
/**
 * ActiveTxCache
 */
class ActiveTxCache : boost::noncopyable {
public:

    typedef boost::shared_ptr<model::Job>      JobPtr; 
    typedef boost::shared_ptr<model::File>     FilePtr; 
    typedef boost::shared_ptr<model::Transfer> TransferPtr; 

    /**
     * Cache Entry Type
     */
    struct TransferEntry{
        
        /*
         * The embedded object
         */
        TransferPtr tx;
        FilePtr     file;
        JobPtr      job;

        /*
         * Constructor
         */        
        TransferEntry(TransferPtr t, FilePtr f, JobPtr j):
            tx(t),file(f),job(j){}
    };
    
    /**
     * Cache Entry Vector
     */
    typedef std::vector<TransferEntry> TransferEntryArray;
    
    /**
     * Request Entry Type
     */
    struct RequestEntry{
        std::string id;
        TransferEntryArray transfers;
        /**
         * Constructor
         */
        RequestEntry(const std::string& i):id(i){};
    };
    
    /**
     * Request Entry Vector
     */
    typedef std::vector<RequestEntry> RequestEntryArray;
    
    /**
     * Constructor
     */
    ActiveTxCache();
        
    /**
     * Destructor
     */
    ~ActiveTxCache();

    /**
     * Set Flag
     */
    bool isSet() const {return m_isSet;}
    
    /**
     * Insert a Transfer into the Cache
     */
    void insert(TransferPtr tx, FilePtr file, JobPtr job) /*throw (LogicError)*/;

    /**
     * Get a Transfer cache entry
     */
    TransferEntry get(const std::string& file_id) const /*throw (RuntimeError)*/;
    /**
     * Get a Job Ptr (if exists)
     */
    JobPtr getJob(const std::string& job_id) const /*throw (RuntimeError)*/;

    /**
     * Get a File Ptr (if exists)
     */
    FilePtr getFile(const std::string& file_id) const /*throw (RuntimeError)*/;

    /**
     * Get the Transfer cache entry that refers to a request
     */
    RequestEntry getRequest(const std::string& req_id) const /*throw (RuntimeError)*/;

    /**
     * Get all the Transfer cache entries
     */
    TransferEntryArray get() const /*throw ()*/;

    /**
     * Get the request entries
     */
    RequestEntryArray getRequests() const /*throw ()*/;

    /**
     * Remove a Transfer from the cache
     */
    void remove(const std::string& file_id) /*throw ()*/;

    /**
     * Reset the Cache
     */
    void reset() /*throw ()*/;

    /**
     * Count the number of entries in the cache
     */
    unsigned int count() const /*throw ()*/;
    /**
     * Count the number of entries in the cache for a given VO
     */
    unsigned int count(const std::string& vo_name) const /*throw ()*/;

    /**
     * Count the number of requests in the cache
     */
    unsigned int countRequests() const /*throw ()*/;
    /**
     * Count the number of requests in the cache for a given VO
     */
    unsigned int countRequests(const std::string& vo_name) const /*throw ()*/;
    
    /**
     * Count the number of transfers in the cache for the given source SE
     */
    unsigned int countPerSourceSE(const std::string& src_se) const /*throw ()*/;

    /**
     * Count the number of transfers in the cache for the given destination SE
     */
    unsigned int countPerDestSE(const std::string& dst_se) const /*throw ()*/;

private:
    /**
     * Pimpl
     */
    boost::scoped_ptr<class ActiveTxCacheImpl> m_impl;
    /**
     * Set Flag
     */
    bool m_isSet;
};

} // end namespace channel
} // end namespace action
} // end namespace agent
} // end namespace transfer
} // end namespace data
} // end namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTIONS_CHANNEL_ACTIVETXCACHE_H_
