/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Transfer Agents: Channel BaseAction Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: ChannelAction.h,v 1.6 2006/08/14 08:37:40 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_3_0_2_2 $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_CHANNELACTION_H_
#define GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_CHANNELACTION_H_

#include "glite/data/transfer/agent/action/Action.h"
#include "glite/data/transfer/agent/action/channel/ActiveTxCache.h"
#include "glite/data/transfer/agent/action/ExecuteException.h"
#include "glite/data/transfer/agent/dao/channel/JobDAO.h"
#include "glite/data/transfer/agent/dao/channel/FileDAO.h"
#include "glite/data/transfer/agent/dao/channel/TransferDAO.h"
#include "glite/data/transfer/agent/dao/channel/ChannelDAO.h"
#include "glite/data/transfer/agent/ts/TransferService.h"
#include "glite/data/agents/dao/DAOContext.h"

#include <string>
#include <boost/scoped_ptr.hpp>
#include <set>

namespace log4cpp{
    class Category;
}

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace action  {
namespace channel {
        
/**
 * Base Class For all the VO Actions
 */
class ChannelAction : public Action {
public:    
    
    /**    
     * set DAOContext
     */
    virtual void context(const std::string& channel_name, agents::dao::DAOContext& ctx);
    
    /**    
     * Get DAOContext
     */
    virtual agents::dao::DAOContext& context() /*throw (LogicError)*/;
    
    /**
     * Destructor
     */
    virtual ~ChannelAction();

    /**
     * Execute The Action
     */
    virtual void execute(const std::string& params = "") /*throw (LogicError,ExecuteException)*/;

protected:

    /**
     * Constructor
     */
    ChannelAction(const std::string& name);

    /**
     * Call the virtual method to execute the action (Template Pattern)
     */
    virtual void exec(const std::string& params) /*throw (LogicError, DAOException, ExecuteException)*/ = 0;

    /**
     * Fill Transfer Cache
     */
    void fillTransfersCache() /* throw(DAOException,LogicError)*/;

    /**
     * Update Active Transfers Cache
     */
    void updateTransfersCache(std::set<std::string>& jobIds) /* throw(DAOException,LogicError)*/;

    /**
     * Update the status of the file and Transfer instances related to an 
     * Active Transfer Entry in the Cache
     */
    void updateTransfer(ActiveTxCache::TransferEntry& entry, std::set<std::string>& jobIds) /* throw(DAOException,LogicError)*/ ;

    /*
     * failRequest
     *
     * Fail all the files in a Request
     */
    void failRequest(ActiveTxCache::RequestEntry& req, model::Error::Category reason_class, const std::string& reason, const std::string& proxy_file, bool abort_request, std::set<std::string>& jobIds) /* throw(DAOException,LogicError)*/;

    /**
     * Update Jobs Status
     */
    void updateJobs(const std::set<std::string>& ids) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Update Jobs' State
     */
    void updateJobs(const agents::dao::StringArray& ids) /*throw (DAOLogicError, DAOException)*/;

    /**
     * Send a Fail Event for all the files
     */
    void failAllFiles(const agents::dao::StringArray& files, model::Error::Category c, const std::string& reason) /*throw ()*/;
    /**
     * Send a Fail Event for all the files
     */
    void failAllFiles(const agents::dao::StringArray& files, model::Error::Category c, const std::string& reason,std::set<std::string>& ids) /*throw ()*/;

    /**
     * Get the proxy certificate to be used for a job action
     */
    void getProxyForJob(const std::string& job_id, std::string& proxy_file) /* throw(AgentException)*/;

    /**
     * Get the proxy certificate to be used for a job action
     */
    void getProxyForJob(model::Job& job, std::string& proxy_file) /* throw (AgentException)*/;

    /**
     * Normalize a Surl accordingly with the VO policy
     */
    std::string normalizeSurl(const std::string& surl, const std::string& srm_endpoint) /* throw()*/;

protected:
    
    typedef dao::channel::JobDAO       JobDAO;
    typedef dao::channel::FileDAO      FileDAO;
    typedef dao::channel::TransferDAO  TransferDAO;
    typedef dao::channel::ChannelDAO   ChannelDAO;
    typedef ts::TransferService        TransferService;

    /**    
     * Get a JobDAO Instance
     */
    JobDAO& jobDAO() /*throw (ExecuteException)*/;
    
    /**    
     * Get a FileDAO Instance
     */
    FileDAO& fileDAO() /*throw (ExecuteException)*/;

    /**    
     * Get a TransferDAO Instance
     */
    TransferDAO& transferDAO() /*throw (ExecuteException)*/;

    /**    
     * Get a ChannelDAO Instance
     */
    ChannelDAO& channelDAO() /*throw (ExecuteException)*/;

    /**    
     * Get a Transfer Service Instance
     */
    TransferService& transferService() /*throw (ExecuteException)*/;
    
    /**
     * The Logger
     */
    mutable class log4cpp::Category& m_logger;

    /**
     * Get the Channel Name
     */
    const std::string& channelName() const {return m_channelName;}

private:
    
    /**
     * Default Constructor. Not Implemented
     */
    ChannelAction();

private:
    
    /**
     * The DAO Context
     */
    agents::dao::DAOContext* m_context;

    /**
     * The Channel Name
     */
    std::string m_channelName;

    /**
     * The Job DAO
     */
    boost::scoped_ptr<JobDAO> m_jobDAO;

    /**
     * The File DAO
     */
    boost::scoped_ptr<FileDAO> m_fileDAO;
    
    /**
     * The Transfer DAO
     */
    boost::scoped_ptr<TransferDAO> m_transferDAO;
    
    /**
     * The Channel DAO
     */
    boost::scoped_ptr<ChannelDAO> m_channelDAO;
    
    /**    
     * The TransferService Instance
     */
    boost::scoped_ptr<TransferService> m_transferService;
    
};

} // End namespace channel
} // End namespace action
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_ACTION_CHANNEL_CHANNELACTION_H_
