/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Transfer Agents: CatalogService Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CatalogService.h,v 1.4 2006/03/06 12:56:28 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_3_0_2_2 $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICE_H_
#define GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICE_H_

#include "glite/data/transfer/agent/catalog/CatalogServiceException.h"
#include <vector>
#include <string>
#include <boost/shared_ptr.hpp>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace catalog {

/**
 * CatalogService Interface.
 * Define the interface for registering and retrieving logical and physical 
 * names from a catalog Service
 */
class CatalogService{
public:
    /**
     * FileStat base class. The implementation of the CatalogService interface
     * should extend this class and provide the stat information they need in 
     * order to syncronized the source and destination catalogs, e.g. 
     * permissions, metadata, etc
     */
    struct FileStat : boost::noncopyable{
        /**
         * Logical Name
         */
        std::string logical;

        /**
         * Desturctor
         */
        virtual ~FileStat(){}
    };

    typedef boost::shared_ptr<FileStat>        FileStatPtr;
    typedef std::vector<FileStatPtr>           FileStatArray;
    typedef std::pair<std::string,std::string> StringPair;
    typedef std::vector<std::string>           StringArray;
    typedef std::vector<StringPair>            StringPairArray;

    /**
     * Replica Structure
     */
    struct ReplicaStat{
        /**
         * Logical Name
         */
        std::string logical;
        /**
         * SURL
         */
        std::string surl;
        /**
         * SURL
         */
        FileStatPtr stat;
        /**
         * Constructors
         */
        ReplicaStat(){}
        ReplicaStat(const std::string& l, const std::string& s):logical(l),surl(s){}
        ReplicaStat(const std::string& l, const std::string& s, FileStatPtr p):logical(l),surl(s),stat(p){}
    };

    typedef std::vector<ReplicaStat>         ReplicaStatArray;
    
    /**
     * Destructor
     */
    virtual ~CatalogService(){}
        
    /**
     * CatalogResult Class.
     */
    struct CatalogResult{
        /**
         * ResultStatus
         */
        typedef enum{
            R_SUCCESS       = 0,
            R_SOME_FAILURES = -1,
            R_FAILED        = -2
        } ResultStatus;
        
        /**
         * Overall Request Result
         */
        ResultStatus status;
        /**
         * Overall Result Reason (in case of failures)
         */
        std::string reason;
        /**
         * Array holding the pair <Failed file logical name - related reason>
         */
        StringPairArray failures;
    };
    
    /**
     * Check that the user is allowed to read and replicate the requested files
     * @param logicals [IN] The list of input Logical Name to be checked. It can
     * be either LFN or GUID, but the same type should be used within a request 
     * @param result [OUT] The result of the call. In case of failures, it may
     * contain the list of the logical name of the files taht failed. 
     * @throws CatalogRemoteError in case of errors contacting the remote Catalog.
     * @throws CatalogLogicError in case of errors in the Catalog client logic
     * (e.g. failed precondition checks)
     */
    virtual void checkPermissions(const StringArray& logicals, 
                                        CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/ = 0;
    
    /**
     * List the names of all the replicas associated to the given Logical Name
     * @param logicals [IN] The list of input Logical Name.  It can be 
     * either LFN or GUID, but the same type should be used within a request 
     * @param site [IN] The site name that can be used to fileter the request.
     * the usage of this parameter should be intended as a hint and therefore
     * is not compulsory that the returned SURLs are only belonging to the site
     * @param surls [OUT] the array that will store the name of all the replicas
     * @param result [OUT] The result of the call. In case of failures, it may
     * contain the list of the logical name of the files taht failed. In case the 
     * result returned that only some files have failed, the "surls" parameter
     * should contains an empty value corresponding to the failed logical name
     * @throws CatalogRemoteError in case of errors contacting the remote Catalog.
     * @throws CatalogLogicError in case of errors in the Catalog client logic
     * (e.g. failed precondition checks)
     */
    virtual void listSurls(const StringArray& logicals, const std::string& site,
                                 std::vector<StringArray>& surls,
                                 CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/ = 0;

    /**
     * Check that the given SURLs don't already exist 
     * @param name [IN] The list of Logical Name - SURL pairs be be 
     * checked. The LFN can be either an LFN or GUID, but the same type should 
     * be used within the same  request 
     * @param result [OUT] The result of the call. In case of failures, it may
     * contain the list of the logical name of the files taht failed. 
     * @throws CatalogRemoteError in case of errors contacting the remote Catalog.
     * @throws CatalogLogicError in case of errors in the Catalog client logic
     * (e.g. failed precondition checks)
     */
    virtual void checkSurls(const StringPairArray& names, 
                                  CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/ = 0;
    
    /**
     * Get the Stats info of a files. This information would the be 
     * passed to the destination catalog in the registerSurls method. The purpose
     * of this method is to synchronize permisisons information between two 
     * catalogs. This method may not be called in case the source and 
     * destination catalog endpoint are the same
     * @param name [IN] The list of input Logical Name - SURL pair. The logical 
     * name can be either LFN or GUID, but the same type should be used within 
     * the same request 
     * @param stats [OUT] the array that will store the FileStat information
     * @param result [OUT] The result of the call. In case of failures, it may
     * contain the list of the logical name of the files taht failed. In case the 
     * result returned that only some files have failed, the "stats" parameter
     * should contains an empty value corresponding to the failed logical name
     * @throws CatalogRemoteError in case of errors contacting the remote Catalog.
     * @throws CatalogLogicError in case of errors in the Catalog client logic
     * (e.g. failed precondition checks)
     */
    virtual void getStats(const StringPairArray& names, FileStatArray& stats, 
                                CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/ = 0;

    /**
     * Register the name of new replicas 
     * @param replicas [IN] The list of The Logical Name, SURL and the related 
     * stats. The Logical Name can be either LFN or GUID, but the same type 
     * should be used within a request. In case the source and the destination
     * catalog are the same, the stat information may be empty
     * @param result [OUT] The result of the call. In case of failures, it may
     * contain the list of the logical name of the files that failed.
     * @throws CatalogRemoteError in case of errors contacting the remote Catalog.
     * @throws CatalogLogicError in case of errors in the Catalog client logic
     * (e.g. failed precondition checks)
     */
    virtual void registerSurls(const ReplicaStatArray& replicas,
                                     CatalogResult& result) /*throw (CatalogLogicError, CatalogRemoteError)*/ = 0;
                               
protected:
    /**
     * Constructor
     */
    CatalogService(){}

private:
    /** @link dependency
     * @label throws*/
    /*# CatalogLogicError lnkCatalogLogicError; */

    /** @link dependency
     * @label throws*/
    /*# CatalogRemoteError lnkCatalogRemoteError; */
};

} // End namespace catalog
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICE_H_
