/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Transfer Agents: CatalogServiceFactory Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: CatalogServiceFactory.h,v 1.5 2006/08/14 08:37:42 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_3_0_2_2 $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICEFACTORY_H_
#define GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICEFACTORY_H_

#include "glite/data/transfer/agent/catalog/CatalogServiceException.h"
#include "glite/data/transfer/agent/catalog/CatalogService.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace catalog {

/**
 * CatalogServiceFactory interface
 * @interface
 */
class CatalogServiceFactory  {
public:

    /**
     * Create a CatalogService instance
     * @param endpoint [IN] the endpoint of the service to contact 
     * @param type [IN] the type of the service to create (can be the value 
     * returned either by globalCatalogServiceType or localCatalogServiceType
     * methods)
     * @param cred [IN] the name of the file containing the credential to be
     * used to contact the service. If this value is empty, the default will 
     * apply
     * @return a new instance of a CatalogService
     * @throws CatalogLogicError in the type is not supported
     */
    virtual CatalogService * create(const std::string& endpoint, const std::string& type, const std::string& cred = "") /*throw (CatalogLogicError)*/ = 0; 

    /**
     * Return the Global Catalog Service Type (if any) to be used to query 
     * ServiceDiscovery. If the Catalog Deployment model doesn't support the
     * global scenario, an emty string should be returned
     * @return the Global Catalog Service Type to be used to query 
     */
    virtual std::string globalCatalogServiceType() /*throw ()*/ = 0; 

    /**
     * Return the Local Catalog Service Type (if any) to be used to query 
     * ServiceDiscovery. If the Catalog Deployment model doesn't support the
     * local scenario, an emty string should be returned
     * @return the local Catalog Service Type to be used to query 
     */
    virtual std::string localCatalogServiceType() /*throw ()*/ = 0; 

    /**
     * Return the endpoint and the type of the CatalogService to use. The 
     * default implementation uses in the information system and apply the 
     * following rules: first looks up in a Catalog of the type returned by 
     * localCatalogServiceType exists on the given site; if not retunr the first
     * instance of the type returned by globalCatalogServiceType
     * @param site [IN] the site where the calaogs should be located (if not 
     * global)
     * @param vo_name [IN] the name of the VO of the user that needs to access
     * the catalog
     * @return the endpoint and the type of the CatalogService to use
     * @throws LogicError in case of errors in retrieving the 
     * Catalog Endpoint
     */
    virtual std::pair<std::string,std::string> getCatalogEndpoint(const std::string& site, const std::string& vo_name) /* throw(LogicError)*/;
 
    /**
     * Return the configured CatalogServiceFactory Instance. Please note that 
     * this method doesn't allocate the instance (it's not a singleton)
     * @return the configued CatalogServiceFactory 
     * @throws LogicError in case of no CatalogServiceFactory is
     * configured
     */
    static CatalogServiceFactory& instance() /*throw (LogicError)*/; 
   
protected:

    /**
     * Constructor
     */ 
    CatalogServiceFactory();

    /**
     * Destructor
     */ 
    virtual ~CatalogServiceFactory();

private:

    /**
     * The registered Instance
     */
    static CatalogServiceFactory * s_instance;
    
    /** @link dependency 
     * @label creates*/
    /*# CatalogService lnkCatalogService; */

    /** @link dependency 
     * @label throws*/
    /*# CatalogServiceException lnkCatalogServiceException; */
};

} // End namespace catalog
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_CATALOG_CATALOGSERVICEFACTORY_H_
