/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Transfer Agents: Channel Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Channel.h,v 1.7 2007/02/15 18:10:15 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_3_0_2_2 $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_MODEL_CHANNEL_H_
#define GLITE_DATA_TRANSFER_AGENTS_MODEL_CHANNEL_H_

#include "glite/data/transfer/agent/model/Property.h"
#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace model   {

/**
 * Channel Class
 * @stereotype ValueObject
 */
struct Channel {

    typedef enum{ // Define values so they could combined in OR
        S_UNDEF        = 0x0000,
        S_ACTIVE       = 0x0001,
        S_DRAIN        = S_ACTIVE     << 1,  //0x0002
        S_INACTIVE     = S_DRAIN      << 1,  //0x0004
        S_STOPPED      = S_INACTIVE   << 1,  //0x0008
        S_HALTED       = S_STOPPED    << 1,  //0x0010
        S_ARCHIVED     = S_HALTED     << 1   //0x0020
    } State;

    typedef enum{
        VS_UNDEF    = 0x0000,
        VS_ON       = 0x0001,
        VS_OFF      = VS_ON   << 1,  //0x0002
        VS_PAUSE    = VS_OFF  << 1,  //0x0004
    } VOState;

    typedef enum{
        T_UNDEF         = 0x0000,
        T_DEDICATED     = 0x0001,
        T_NON_DEDICATED = T_DEDICATED   << 1,  //0x0002
    } Type;
    
    /**
     * Default Constructor
     */
    Channel():state(S_UNDEF),files(0),streams(0),bandwidth(0.0),
            throughput(0.0),lastActive((time_t)-1),lastModification((time_t)-1),
            seLimit(0),type(T_DEDICATED){}

    /**
     * Constructor
     */
    Channel(const std::string& n,         // channel_name
                  State        s,         // channel_state
            const std::string& ssite,     // source_site
            const std::string& dsite,     // dest_site
                  unsigned int files,     // num_of_files
                  unsigned int strms = 0, // num_of_streams
            const std::string& tbs = "",  // tcp_buffer_size
                  double       b = 0,     // bandwidth
                  double       t = 0,     // nominal_throughput
            const std::string& c = "",    // contact
            const std::string& msg = "",  // message
            const std::string& a_dn = "", // admin_dn
                  time_t l = (time_t)-1,  // last_active
                  time_t lm = (time_t)-1, // last_modification
                  unsigned int sel = 0,   // se_limit
                  Type  ct = T_DEDICATED):// channel_type
            name(n),state(s),sourceSite(ssite),destSite(dsite),
            files(files),streams(strms),tcpBS(tbs), bandwidth(b),throughput(t),
            contact(c),message(msg),adminDn(a_dn),
            lastActive(l),lastModification(lm),
            seLimit(sel),type(ct){}

    /**
     * Name Property
     */
    Property<std::string> const name;

    /**
     * State Property
     */
    Property<State> state;

    /**
     * SourceSite Property
     */
    Property<std::string> const sourceSite;

    /**
     * DestSite Property
     */
    Property<std::string> const destSite;

    /**
     * Number of Files Property
     */
    Property<unsigned int> const files;
        
    /**
     * Number of Streams Property
     */
    Property<unsigned int> const streams;

    /**
     * TCP Buffer Size Property
     */
    Property<std::string> const tcpBS;

    /**
     * Bandwidth Property
     */
    Property<double> const bandwidth;
        
    /**
     * Nominal Throughput Property
     */
    Property<double> const throughput;
    
    /**
     * Contact Property
     */
    Property<std::string> const contact;
    
    /**
     * Message Property
     */
    Property<std::string> const message;
    
    /**
     * AdminDN Property
     */
    Property<std::string> const adminDn;
    
    /**
     * LastActive Property
     */
    Property<time_t> lastActive;

    /**
     * Last Admin Modification Property
     */
    Property<time_t> const lastModification;

    /**
     * SE Limit Property
     */
    Property<unsigned int> const seLimit;

    /**
     * Channel Type Property
     */
    Property<Type> const type;

public:
    /**
     * Reset All R/W Properties
     */
    void reset(){
        state.reset();
        lastActive.reset();
    }
};

} // End namespace model
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_MODEL_CHANNEL_H_
