/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Transfer Agents: File Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: File.h,v 1.8 2007/02/15 18:10:15 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_3_0_2_2 $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_MODEL_FILE_H_
#define GLITE_DATA_TRANSFER_AGENTS_MODEL_FILE_H_

#include "glite/data/agents/glite-types.h"
#include "glite/data/transfer/agent/model/Property.h"
#include "glite/data/transfer/agent/model/Error.h"
#include <string>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace model   {
    
/**
 * File Class
 * @stereotype ValueObject
 */
struct File {

    typedef enum{ // Define values so they could combined in OR
        S_UNDEF                        = 0x0000,
        S_SUBMITTED                    = 0x0001,
        S_PENDING                      = S_SUBMITTED                    << 1,  //0x0002
        S_READY                        = S_PENDING                      << 1,  //0x0004
        S_ACTIVE                       = S_READY                        << 1,  //0x0008
        S_DONE                         = S_ACTIVE                       << 1,  //0x0010
        S_WAITING                      = S_DONE                         << 1,  //0x0020
        S_HOLD                         = S_WAITING                      << 1,  //0x0040
        S_FAILED                       = S_HOLD                         << 1,  //0x0080
        S_FINISHING                    = S_FAILED                       << 1,  //0x0100
        S_FINISHED                     = S_FINISHING                    << 1,  //0x0200
        S_AWAITING_PRESTAGE            = S_FINISHED                     << 1,  //0x0400
        S_PRESTAGING                   = S_AWAITING_PRESTAGE            << 1,  //0x0800
        S_WAITING_PRESTAGE             = S_PRESTAGING                   << 1,  //0x1000
        S_WAITING_CATALOG_RESOLUTION   = S_WAITING_PRESTAGE             << 1,  //0x2000
        S_WAITING_CATALOG_REGISTRATION = S_WAITING_CATALOG_RESOLUTION   << 1,  //0x4000
        S_CANCELED                     = S_WAITING_CATALOG_REGISTRATION << 1,  //0x8000
    } State;

    /**
     * Default Constructor
     */
    File():state(S_UNDEF),reasonClass(Error::UNDEF),fileSize(-1),
           failures(0),currentFailures(0),catalogFailures(0),
           prestageFailures(0),finishTime((time_t)-1){}

    /**
     * Constructor
     */
    File(const std::string& i,                     // file_id
         const std::string& j_id,                  // job_id
               State        s,                     // file_state
         const std::string& l      = "",           // logical_name
         const std::string& ssurl  = "",           // source_surl
         const std::string& dsurl  = "",           // dest_surl
               Error::Category rc  = Error::UNDEF, // reason_class
         const std::string& r      = "",           // reason
               unsigned int fl     = 0,            // failures
               unsigned int cf     = 0,            // current_failures
               unsigned int cat_f  = 0,            // catalog_failures
               unsigned int ps_f   = 0,            // prestage_failures
               agents::longlong fs = -1,           // filesize
         const std::string& cksum  = "",           // check_sum
               time_t       ft     = (time_t)-1,   // finish_time
         const std::string& dn     = "",           // agent_dn
         const std::string& i_p    = ""):          // internal_params
                 id(i),jobId(j_id),state(s),logical(l),
                 sourceSurl(ssurl),destSurl(dsurl),reasonClass(rc),
                 reason(r),failures(fl),currentFailures(cf),
                 catalogFailures(cat_f),prestageFailures(ps_f),fileSize(fs),
                 checksum(cksum),finishTime(ft),agentDn(dn),
                 internalParams(i_p){}

    /**
     * Id Property
     */
    Property<std::string> const id;

    /**
     * Job Id Property
     */
    Property<std::string> const jobId;

    /**
     * State Property
     */
    Property<State> state;

    /**
     * Logical Name Property
     */
    Property<std::string> const logical;

    /**
     * Source SURL Property
     */
    Property<std::string> sourceSurl;

    /**
     * Dest SURL Property
     */
    Property<std::string> destSurl;
    
    /**
     * ReasonClass Property
     */
    Property<Error::Category> reasonClass;

    /**
     * Reason Property
     */
    Property<std::string> reason;
    
    /**
     * Failures Property: sum of all the failures (transfer, catalog, prestage)
     */
    Property<unsigned int> failures;

    /**
     * Current Failures Property (reset at the Hold->Pending transition)
     * Please note that this property count only the transfer Failures
     */
    Property<unsigned int> currentFailures;

    /**
     * Catalog Failures Property (not reset at the Hold->Pending transition, 
     * since the file can't go to Hold after catalog failures)
     */
    Property<unsigned int> catalogFailures;

    /**
     * Prestage Failures Property (reset at the Hold->Pending transition)
     */
    Property<unsigned int> prestageFailures;
    
    /**
     * File Size Property
     */
    Property<agents::longlong> fileSize;

    /**
     * Checksum Property
     */
    Property<std::string> checksum;
    
    /**
     * Finish Time Property
     */
    Property<time_t> finishTime;

    /**
     * Agent DN Property
     */
    Property<std::string> agentDn;

    /**
     * File Internal Params Property
     */
    Property<std::string> internalParams;

public:
    /**
     * Reset All R/W Properties
     */
    void reset(){
        state.reset();
        sourceSurl.reset();
        destSurl.reset();
        reasonClass.reset();
        reason.reset();
        failures.reset();
        currentFailures.reset();
        catalogFailures.reset();
        prestageFailures.reset();
        fileSize.reset();
        checksum.reset();
        finishTime.reset();
        agentDn.reset();
        internalParams.reset();
    }
};

} // End namespace model
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_MODEL_FILE_H_
