/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Transfer Agents: Transfer Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: Transfer.h,v 1.7 2007/02/15 18:10:15 badino Exp $
 *  Release: $Name: glite-data-transfer-agents_R_3_0_2_2 $
 *
 */
#ifndef GLITE_DATA_TRANSFER_AGENTS_MODEL_TRANSFER_H_
#define GLITE_DATA_TRANSFER_AGENTS_MODEL_TRANSFER_H_

#include <string>
#include "glite/data/agents/glite-types.h"
#include "glite/data/transfer/agent/model/Error.h"
#include "glite/data/transfer/agent/model/Property.h"

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace model   {
    
/**
 * Transfer Class
 * @stereotype ValueObject
 */
struct Transfer {

    typedef enum{ // Define values so they could combined in OR
        S_UNDEF           = 0x0000,
        S_RECEIVED        = 0x0001,
        S_PREPARING       = S_RECEIVED      << 1,  //0x0002
        S_READY           = S_PREPARING     << 1,  //0x0004
        S_SUBMITTED       = S_READY         << 1,  //0x0008
        S_PROCESSING      = S_SUBMITTED     << 1,  //0x0010
        S_TRANSFERRED     = S_PROCESSING    << 1,  //0x0020
        S_COMPLETED       = S_TRANSFERRED   << 1,  //0x0040
        S_FAILED          = S_COMPLETED     << 1,  //0x0080
        S_ABORTED         = S_FAILED        << 1   //0x0100
    } State;

    /**
     * Default Constructor
     */
    Transfer():transferId(0),
               state(S_UNDEF),
               transferTime((time_t)-1),prepareTime((time_t)-1),
               duration(0.0),srcPrepDuration(0),dstPrepDuration(0),
               txDuration(0),srcFinalDuration(0),dstFinalDuration(0),
               bytesWritten(-1),throughput(0.0),reasonClass(Error::UNDEF),
               fileSize(0){}
    /**
     * Constructor
     */
    Transfer(const std::string&     req_id,               // request_id
             const std::string&     file_id,              // file_id
                   unsigned int     t_id,                 // transfer_id
             const std::string&     j_id,                 // job_id
                   State            s,                    // state
             const std::string&     ttype = "",           // transfer_type
             const std::string&     sturl = "",           // source_turl
             const std::string&     dturl = "",           // dest_turl
                   time_t           t     = (time_t)-1,   // transfer_time
                   time_t           p_t   = (time_t)-1,   // prepare_time
                   double           d     = 0.0,          // duration
                   double           s_p_d = 0.0,          // src_preparation_duration
                   double           d_p_d = 0.0,          // dest_preparation_duration
                   double           t_d   = 0.0,          // transfer_duration
                   double           s_f_d = 0.0,          // src_finalization_duration
                   double           d_f_d = 0.0,          // dest_finalization_duration
                   agents::longlong bw    = -1,           // bytes_written
                   double           th    = 0.0,          // throughput
                   Error::Category  rc    = Error::UNDEF, // reason_class
             const std::string&     r     = "",           // reason
                   agents::longlong fs    = 0,            // file_size
             const std::string&     s_tkn = "",           // src_srm_token
             const std::string&     d_tkn = "",           // dest_srm_token
             const std::string&     s_h   = "",           // src_host
             const std::string&     d_h   = ""):          // dest_host
                 requestId(req_id),fileId(file_id),transferId(t_id),jobId(j_id),
                 state(s),transferType(ttype),sourceTurl(sturl),destTurl(dturl),
                 transferTime(t),prepareTime(p_t),duration(d),
                 srcPrepDuration(s_p_d),dstPrepDuration(d_p_d),txDuration(t_d),
                 srcFinalDuration(s_f_d),dstFinalDuration(d_f_d),
                 bytesWritten(bw),throughput(th),reasonClass(rc),reason(r),
                 fileSize(fs),
                 srcSrmToken(s_tkn),destSrmToken(d_tkn),srcHost(s_h),destHost(d_h){}

    /**
     * RequestId Property
     */
    Property<std::string> const requestId;

    /**
     * FileId Property
     */
    Property<std::string> const fileId;

    /**
     * TransferId Property
     */
    Property<unsigned int> const transferId;
                     
    /**
     * JobId Property
     */
    Property<std::string> const jobId;

    /**
     * State Property
     */
    Property<State> state;

    /**
     * Dest TURL Property
     */
    Property<std::string> const transferType;

    /**
     * Source TURL Property
     */
    Property<std::string> sourceTurl;

    /**
     * Dest TURL Property
     */
    Property<std::string> destTurl;

    /**
     * Transfer Time Property
     */
    Property<time_t> transferTime;

    /**
     * Prepare Time Property
     */
    Property<time_t> prepareTime;

    /**
     * Transfer Duration Property
     */
    Property<double> duration;

    /**
     * Duration of the Source file Preparation Property
     */
    Property<double> srcPrepDuration;

    /**
     * Duration of the Destination file Preparation Property
     */
    Property<double> dstPrepDuration;

    /**
     * Duration of the Transfer Property
     */
    Property<double> txDuration;
    
    /**
     * Duration of the Source file Finalization Property
     */
    Property<double> srcFinalDuration;

    /**
     * Duration of the Destination file Finalization Property
     */
    Property<double> dstFinalDuration;

    /**
     * Bytes Written Property
     */
    Property<agents::longlong> bytesWritten;

    /**
     * Average throughput Property
     */
    Property<double> throughput;

    /**
     * ReasonClass Property
     */
    Property<Error::Category> reasonClass;

    /**
     * Reason Property
     */
    Property<std::string> reason;

    /**
     * File Size Property
     */
    Property<agents::longlong> fileSize;

    /**
     * Source SRM Token Property
     */
    Property<std::string> srcSrmToken;

    /**
     * Dest SRM Token Property
     */
    Property<std::string> destSrmToken;

    /**
     * Host for source TURL
     */
    Property<std::string> srcHost;

    /**
     * Host for dest TURL
     */
    Property<std::string> destHost;

public:

    /**
     * Reset All R/W Properties
     */
    void reset(){
        state.reset();
        sourceTurl.reset();
        destTurl.reset();
        transferTime.reset();
        prepareTime.reset();
        duration.reset();
        srcPrepDuration.reset();
        dstPrepDuration.reset();
        txDuration.reset();
        srcFinalDuration.reset();
        dstFinalDuration.reset();
        bytesWritten.reset();
        throughput.reset();
        reasonClass.reset();
        reason.reset();
        fileSize.reset();  
        srcSrmToken.reset();  
        destSrmToken.reset();  
        srcHost.reset();  
        destHost.reset();  
    }
};

} // End namespace model
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_MODEL_TRANSFER_H_
