/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data Transfer Agents: TransferService Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: TransferService.h,v 1.10 2007/02/21 22:31:51 badino Exp $ 
 *  Release: $Name: glite-data-transfer-agents_R_3_0_2_2 $
 */

#ifndef GLITE_DATA_TRANSFER_AGENTS_TS_TRANSFERSERVICE_H_
#define GLITE_DATA_TRANSFER_AGENTS_TS_TRANSFERSERVICE_H_

#include "glite/data/agents/glite-types.h"
#include "glite/data/transfer/agent/model/File.h"
#include "glite/data/transfer/agent/model/Transfer.h"
#include "glite/data/transfer/agent/model/Error.h"
#include "glite/data/transfer/agent/ts/TransferException.h"

#include <vector>
#include <boost/utility.hpp>

namespace glite   {
namespace data    {
namespace transfer{
namespace agent   {
namespace ts      {

/**
 * TransferService interface. This class defines the interface to integrate 
 * different TransferService implementations (Adapter Pattern)
 * @interface
 */
class TransferService : boost::noncopyable {
public:

    /**
     * TransferRequest Struct
     */
    struct TransferRequest{
        typedef std::pair<std::string,std::string> SurlPair;
        typedef std::vector<SurlPair> SurlPairArray;
        SurlPairArray    surls;
        std::string      parameters;
        unsigned long    streams;
        unsigned long    tcpbs;
        std::string      src_srm;
        std::string      src_srm_version;
        std::string      dest_srm;
        std::string      dest_srm_version;
        std::string      dest_token_descr;
        std::string      dest_space_token;
        bool             overwrite_flag;
        std::string      description;
        
        /**
         * Default Constructor
         */
        TransferRequest();
    };

    /**
     * TransferResponse Struct
     */
    struct TransferResponse{
        std::string               request_id;
        std::vector<unsigned int> transfer_ids;
    };
    
    /**
     * TransferObject Array Type
     */
    typedef std::vector<glite::data::transfer::agent::model::Transfer*> TransferInfoArray;

    /**
     * Destructor
     */
    virtual ~TransferService(){}

    /**
     * Return the TransferService type
     * @return the TransferService type
     */
    virtual std::string type() /*throw ()*/ = 0;

    /**
     * Return the name of the Channel the TransferService is responsible for
     * @return the name of the Channel the TransferService is responsible for
     * @throws LogicError in case the channel name is not set
     */
    virtual std::string channel() /*throw (LogicError)*/ = 0;

    /**
     * Get the number of max concurrent transfer requests the service can support
     * @return the number of max concurrent transfer requests the service can support
     */
    virtual unsigned long max() /*throw ()*/ = 0;

    /**
     * Get the number of max transfers that can be processed in the same request
     * @return number of max transfers that can be processed in the same request
     */
    virtual unsigned long maxRequestSize() /*throw ()*/ = 0;

    /**
     * Submit Transfers
     * @param req [INOUT] the request containing the list of SURLs pair to
     * transfer.
     * @param out [OUT] The result  object containing the request identifier and
     * @param credentials [IN] the credentials to be used to perform the action
     * a list of transfer identifier
     * @throws LogicError or TransferException in case of error
     */
    virtual void submit(const TransferRequest& req, TransferResponse& out, const std::string& credentials) /*throw (LogicError, TransferException)*/ = 0;

    /**
     * Cancel a Pending Transfer request
     * @param requestId [IN] the Identifier of the pending Transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     */
    virtual void revoke(const std::string& requestId, const std::string& credentials) /*throw ()*/ = 0;
    
    /**
     * Abort a Pending Transfer request. This method is intended for performing 
     * a best effort clean up when the user credentials are not available to
     * revoke the transfer
     * @param requestId [IN] the Identifier of the pending Transfer request
     */
    virtual void abort(const std::string& requestId) /*throw ()*/ = 0;
    
    /**
     * Get the status of a FileTransfer request
     * @param requestId [IN] the Identifier of the pending Transfer request
     * @param objs [INOUT] the array that will containing the information for
     * each transfer involved on the request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws LogicError, TransferException in case of error
     */
    virtual void state(const std::string& requestId, TransferInfoArray& objs, const std::string& credentials) /*throw (LogicError, TransferException)*/ = 0;
    
    /**
     * Clean-up the information concerning a completed Transfer 
     * @param requestId [IN] the Identifier of the completed Transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws LogicError in case the requested Transfer request 
     * doesn't exist or is not completed
     */
    virtual void clean(const std::string& requestId, const std::string& credentials) /*throw (LogicError)*/ = 0;

    /**
     * Trace a Transfer
     * @param requestId [IN] the Identifier of the completed Transfer request
     * @param out [IN] the buffer where to store the transfer request
     * @param credentials [IN] the credentials to be used to perform the action
     * @throws LogicError in case the requested Transfer request 
     * doesn't exist or is not completed
     * @throws TransferException otherwise
     */
    virtual void trace(const std::string& requestId, std::ostream& out, const std::string& credentials) /*throw (LogicError, TransferException)*/ = 0;

    /**
     * Get the number of all the pending transfer requests on the channel
     * @return the number of all the pending transfer requests on the channel
     * @throws LogicError or TransferException in case of error
     */
    virtual unsigned long count() /*throw (LogicError, TransferException)*/ = 0;

private:

    /** @link dependency */
    /*# File lnkFile; */

    /** @link dependency 
     * @label throws*/
    /*# TransferException lnkTransferException; */
};

} // End namespace ts
} // End namespace agent
} // End namespace transfer
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_TRANSFER_AGENTS_TS_TRANSFERSERVICE_H_
