#
#  Copyright (c) Members of the EGEE Collaboration. 2004.
#  See http://eu-egee.org/partners/ for details on the copyright holders.
#  For license conditions see the license file or http://eu-egee.org/license.html
#
#  This script implement the basic retry strategy
#
#  Authors: Paolo Badino <paolo.badino@cern.ch>
#  Version info: $Id: basic_retry.py,v 1.7 2006/08/14 08:38:12 badino Exp $
#  Release: $Name: glite-data-transfer-agents_R_3_0_2_2 $
#

import time
import glite.fts
import glite.data.agents.utils

#
# Declare Constants used in the script
#
MODULE_NAME      = "basic_retry"
RETRY_DELAY_TIME = 600 #10 minutes
MAX_FAILURES     = 3

#
# Declare the Retry Version supported by this script
#
def RetryVersion():
    return "1.0"

# Log Functions
VerboseDebug  = False
DebugPriority = glite.data.agents.utils.LogPriority.Debug
def log_debug(msg):
    global DebugPriority 
    glite.data.agents.utils.Log(MODULE_NAME,DebugPriority,msg)
    
def log_info(msg):
    glite.data.agents.utils.LogInfo(MODULE_NAME,msg)

def log_warn(msg):
    glite.data.agents.utils.LogWarn(MODULE_NAME,msg)

def log_error(msg):
    glite.data.agents.utils.LogError(MODULE_NAME,msg)

#
# Apply the Basic Retry Strategy to the given File
#
def Retry(job,file,transfers):
    
    global VerboseDebug
    result = glite.fts.RetryResult.Wait
    
    log_debug("Check if File [%s] - %s should be retried: it has %d transfer(s), %d failure(s), %d current failure(s) - transfer attempts %d" % (file.id,job.id,len(transfers),file.failures,file.currentFailures,len(transfers)))
    if(VerboseDebug == True):
        for tx in transfers:
            log_debug("File has transfer attampt %s:%d" % (tx.requestId,tx.transferId))
        
    # Check File Failures
    if(file.currentFailures >= MAX_FAILURES) :
        # Too Many Failures = > Put it in HOLD State
        log_info("Too many failures (%d) for File [%s]: Move it to HOLD" % (file.currentFailures,file.id) )
        result = glite.fts.RetryResult.Hold;
    else:
        # Count the number of attemps
        log_debug("Already performed %d tries" % (len(transfers)))
        
        delay = RETRY_DELAY_TIME
        
        # Get Current Time
        transfer_time = 0
        current = time.time()
        
        # Check if it's time to resubmit the File
        if(-1 != file.finishTime) :
            # Take the file finish time: it contains the last error
            transfer_time = file.finishTime
        else :
            log_debug("No File finish time set. Consider the last transfer start time as reference")
            if(len(transfers) > 0) :
                # Take the last transfer Time
                transfer_time = transfers[0].transferTime;
            else:
                # File is failed with no transfer attempts. Take the Job 
                # Submission Time
                log_debug("Num Failures: %d - Current %d - Job Submission Time: <%s>" % (file.failures, file.currentFailures, time.asctime(time.gmtime(job.submitTime))))
                if file.failures > 0 :
                    transfer_time = job.submitTime + (file.failures - 1) * delay
                else:
                    transfer_time = job.submitTime
                    
        log_debug("Last Transfer Reference Time: <%s> - CurrentTime: <%s> " % (time.asctime(time.gmtime(transfer_time)), time.asctime(time.gmtime(current))))
        if (current > transfer_time) and ((current - transfer_time) >= delay) :
            # Reschedule the File
            log_info("Reschedule File [%s]. Number of tries already performed %d" % (file.id, len(transfers)))
            result = glite.fts.RetryResult.Retry
        else:
            # Elapsed time not sufficient
            log_debug("File [%s] still in Waiting" % (file.id))

    # return result
    log_debug("Retry Result for File [%s] is %s" % (file.id, result))
    return result;
