#
#  Copyright (c) Members of the EGEE Collaboration. 2004.
#  See http://eu-egee.org/partners/ for details on the copyright holders.
#  For license conditions see the license file or http://eu-egee.org/license.html
#
#  This script implement the smarter retry strategy
#
#  Authors: Paolo Badino <paolo.badino@cern.ch>
#  Version info: $Id: smarter_retry.py,v 1.17 2007/03/28 14:03:24 badino Exp $
#  Release: $Name: glite-data-transfer-agents_R_3_0_2_2 $
#

import re
import os
import time
import string
import glite.fts
import glite.data.agents.utils
import glite.data.agents.sd

################################################################################
#
# Declare Defualt values for configuration parameters used in the script
#
################################################################################
MODULE_NAME                    = "smarter_retry"
RETRY_DELAY_FOR_TIMEOUT_ON_GET = 1800 # 1/2 hour
RETRY_DELAY_FOR_FILE_EXISTS    = 600  # 10 minutes
DEFAULT_RETRY_DELAY            = 600  # 10 minutes
MAX_FAILURES                   = 100  # don't retry after 100 failures
MAX_FILE_EXISTS_FAILURES       = 3    # don't retry after 3 consecutive srmput failures
HOLD_ENABLED                   = True
OVERWRITE_FAILED_FILES         = True
OVERWRITE_EXISTING_FILES       = False
SRM_SERVICE_TYPE               = "SRM"

################################################################################
# PatternList Class: It allows a string to be compared against each expression
# This code is taken from the ASPN Cookbook at http://aspn.activestate.com/ASPN
################################################################################
class PatternList( object ):
    # Constructor
    def __init__(self , patterns = []):
        self.patterns = []
        for p in patterns:
            self.add( p )
    # Add a pettern
    def add( self , pattern ):
        pat = re.compile( pattern )
        self.patterns.append( pat )
    # Check if the pattern is evaluated
    def __contains__(self , item ):
        s = item.lower()
        ret = False
        for p in self.patterns:
            if p.search( s ):
                ret = True
                break
        return ret

################################################################################
# ConfigReader Class: Parse a string with the format of an INI file
# This code is taken from the ASPN Cookbook at http://aspn.activestate.com/ASPN
################################################################################
class ConfigReader:
    #
    # Constructor
    #
    def __init__(self, str):
        self.__hash = {}
        sec = re.compile(r'^\[(.*)\]')
        eq = re.compile(r'^([^=]+)=(.*)')
        str = str.replace(';','\n');
        section = None
        for line in str.split("\n"):
            if sec.search(line):
                lbracket, section, rbracket = sec.split(line)
                section = section.strip() # remove leading and trailing spaces
            elif eq.search(line):
                left, item, value, right = eq.split(line)
                if section is not None:
                    self.__hash[section + '.' + item.strip()] = value.strip()
                else:
                    self.__hash[item.strip()] = value.strip()
    #
    # Get Iteam
    #
    def __getitem__(self, name):
       return self.__hash[name]
    #
    # Check Iteam Key
    #
    def has_key(self, aItem):
       return self.__hash.has_key(aItem)
                                                                                                                                                           

################################################################################
# SmarterRetry Configuration: This class will old all the configuration 
# paramters
################################################################################
class SmarterRetryConfiguration( object ):
    #
    # Constructor
    #
    def __init__(self , config_params = ""):
        # Set the Default Values
        self.ModuleName                   = MODULE_NAME
        self.MaxFailures                  = MAX_FAILURES
        self.MaxFileExistsFailures        = MAX_FILE_EXISTS_FAILURES
        self.HoldEnabled                  = HOLD_ENABLED
        self.OverwriteFailedFiles         = OVERWRITE_FAILED_FILES
        self.OverwriteExistingFiles       = OVERWRITE_EXISTING_FILES
        self.DefaultRetryDelay            = DEFAULT_RETRY_DELAY
        self.RetryDelayForTimeoutOnGet    = RETRY_DELAY_FOR_TIMEOUT_ON_GET
        self.RetryDelayForDestFileExists  = RETRY_DELAY_FOR_FILE_EXISTS
        self.SrmServiceType               = SRM_SERVICE_TYPE
        self.DebugPriority                = glite.data.agents.utils.LogPriority.Debug
        # Read the parameters
        if(len(config_params) > 0):
            reader = ConfigReader(config_params)
            # Get the configuration parameters
            if(reader.has_key('MaxFailures')):
                self.MaxFailures = int(reader['MaxFailures'])
            if(reader.has_key('MaxFileExistsFailures')):
                self.MaxFileExistsFailures = int(reader['MaxFileExistsFailures'])
            if(reader.has_key('HoldEnabled')):
                val = string.lower(reader['HoldEnabled'])
                if((val == "true") or (val == "1")):
                    self.HoldEnabled = True
                else :
                    self.HoldEnabled = False
            if(reader.has_key('OverwriteFailedFiles')):
                val = string.lower(reader['OverwriteFailedFiles'])
                if((val == "true") or (val == "1")):
                    self.OverwriteFailedFiles = True
                else :
                    self.OverwriteFailedFiles = False
            if(reader.has_key('OverwriteExistingFiles')):
                val = string.lower(reader['OverwriteExistingFiles'])
                if((val == "true") or (val == "1")):
                    self.OverwriteExistingFiles = True
                else :
                    self.OverwriteExistingFiles = False
            if(reader.has_key('DefaultRetryDelay')):
                self.DefaultRetryDelay = int(reader['DefaultRetryDelay'])
            if(reader.has_key('RetryDelayForTimeoutOnGet')):
                self.RetryDelayForTimeoutOnGet = int(reader['RetryDelayForTimeoutOnGet'])
            if(reader.has_key('RetryDelayForDestFileExists')):
                self.RetryDelayForDestFileExists = int(reader['RetryDelayForDestFileExists'])
            if(reader.has_key('SrmServiceType')):
                self.SrmServiceType = reader['SrmServiceType']
            if(reader.has_key('DebugAsInfo')):
                val = string.lower(reader['DebugAsInfo'])
                if((val == "true") or (val == "1")):
                    self.DebugPriority = glite.data.agents.utils.LogPriority.Info
                else :
                    self.DebugPriority = glite.data.agents.utils.LogPriority.Debug
        # Log success
        glite.data.agents.utils.LogInfo(self.ModuleName,"Retry Module Initialized. Parameters are:")
        glite.data.agents.utils.LogInfo(self.ModuleName,"MaxFailures                 : %d" % (self.MaxFailures))
        glite.data.agents.utils.LogInfo(self.ModuleName,"MaxFileExistsFailures       : %d" % (self.MaxFileExistsFailures))
        glite.data.agents.utils.LogInfo(self.ModuleName,"HoldEnabled                 : %s" % (str(self.HoldEnabled)))
        glite.data.agents.utils.LogInfo(self.ModuleName,"OverwriteFailedFiles        : %s" % (str(self.OverwriteFailedFiles)))
        glite.data.agents.utils.LogInfo(self.ModuleName,"OverwriteExistingFiles      : %s" % (str(self.OverwriteExistingFiles)))
        glite.data.agents.utils.LogInfo(self.ModuleName,"DefaultRetryDelay           : %d" % (self.DefaultRetryDelay))
        glite.data.agents.utils.LogInfo(self.ModuleName,"RetryDelayForTimeoutOnGet   : %d" % (self.RetryDelayForTimeoutOnGet))
        glite.data.agents.utils.LogInfo(self.ModuleName,"RetryDelayForDestFileExists : %d" % (self.RetryDelayForDestFileExists))
        glite.data.agents.utils.LogInfo(self.ModuleName,"SrmServiceType              : %s" % (self.SrmServiceType))
        if(self.DebugPriority == glite.data.agents.utils.LogPriority.Info):
            glite.data.agents.utils.LogInfo(self.ModuleName,"DebugAsInfo                 : True")
        else:
            glite.data.agents.utils.LogInfo(self.ModuleName,"DebugAsInfo                 : False")

################################################################################
# SmarterRetry Strategy: This class provides the Retry Logic
################################################################################
class SmarterRetryStrategy( object ):
    #
    # Constructor
    #
    def __init__(self , config_params = None):
        # Instantiate the Configuration
        if(config_params is None):
            self.config = SmarterRetryConfiguration()
        else:
            self.config = config_params
        
    # Public Functions
    ############################################################################

    # 
    # apply
    #
    # Evaluate if a file should be retried or not
    #
    def apply(self, job, file, transfers):
        self.debug("Check if File [%s] - %s should be retried: it has %d "\
                   "transfer(s), %d failure(s), %d current failure(s)" \
                   % (file.id,job.id,len(transfers),file.failures,file.currentFailures))
        # Check File Failures
        if(file.currentFailures >= self.config.MaxFailures) :
            # Too Many Failures = > Put it in HOLD State
            self.info("Too many failures (%d) for File [%s] => Don't Retry anymore" \
                     % (file.currentFailures,file.id))
            result = self.dontRetry()
        else:
            # Check the reason of last failure
            if(len(transfers) > 0) :
                last_transfer = transfers[0];
                reason      = last_transfer.reason;
                reasonClass = last_transfer.reasonClass;
            else :
                # No Transfer Attempts
                reason      = file.reason;
                reasonClass = file.reasonClass;
            # Evaluate File based on last Error
            result = self.evaluateReason(reason,reasonClass,job,file,transfers);
        # Return result
        self.debug("Retry Result for File [%s] is %s" % (file.id, result))
        return result;

    # Log Functions
    ############################################################################
    
    #
    # debug
    #
    # Write a debug log message
    #
    def debug(self, msg):
        if(self.config.DebugPriority == glite.data.agents.utils.LogPriority.Info):
            glite.data.agents.utils.LogInfo(self.config.ModuleName,"DEBUG> %s" % msg)
        else :
            glite.data.agents.utils.LogDebug(self.config.ModuleName,msg)
    #
    # info
    #
    # Write an information log message
    #
    def info(self, msg):
        glite.data.agents.utils.LogInfo(self.config.ModuleName,msg)
    #
    # warn
    #
    # Write a warning
    #
    def warn(self, msg):
        glite.data.agents.utils.LogWarn(self.config.ModuleName,msg)
    #
    # error
    #
    # Write an error
    #
    def error(self, msg):
        glite.data.agents.utils.LogError(self.config.ModuleName,msg)
    
    # Private Methods
    ############################################################################
    
    #
    # evaluateReason
    #
    # apply the Retry Logic based on the last error Reason
    #
    def evaluateReason(self, reason, reasonClass, job, file, transfers):
        result = glite.fts.RetryResult.Wait
        # Check the last transfer error reason and reason class
        if((reasonClass == glite.fts.ErrorCategory.Transfer) or \
           (reasonClass == glite.fts.ErrorCategory.Agent)    or \
           (reasonClass == glite.fts.ErrorCategory.Network)  or \
           (reasonClass == glite.fts.ErrorCategory.Remote)):
            # Error During the Transfer SRM 
            result = self.applyForTransferError(reason,reasonClass,job,file,transfers)
        elif((reasonClass == glite.fts.ErrorCategory.Source) or \
             (reasonClass == glite.fts.ErrorCategory.SourcePreparation) or \
             (reasonClass == glite.fts.ErrorCategory.SourceFinalization)):
            # Error on Source SRM 
            result = self.applyForSrmSourceError(reason,reasonClass,job,file,transfers)
        elif((reasonClass == glite.fts.ErrorCategory.Destination) or \
             (reasonClass == glite.fts.ErrorCategory.DestinationPreparation) or \
             (reasonClass == glite.fts.ErrorCategory.DestinationFinalization)):
            # Error on Destination SRM 
            result = self.applyForSrmDestError(reason,reasonClass,job,file,transfers)
        elif(reasonClass == glite.fts.ErrorCategory.Security):
            # Security Error
            result = self.applyForSecurityError(reason,reasonClass,job,file,transfers)
        elif((reasonClass == glite.fts.ErrorCategory.Consistency) or \
             (reasonClass == glite.fts.ErrorCategory.Configuration)):
            # Consistency Or Configuration Error: Don't Retry
            self.info("Consistency or Configuration Error on File [%s] (reason: <%s>) => Don't Retry" \
                      % (file.id,reason) )
            result = self.dontRetry()
        else:
            # Unrecognized Error Category: Don't Retry
            self.warn("Unrecognized Error on File [%s] (reason: <%s>) => Don't Retry" \
                      % (file.id,reason) )
            result = self.dontRetry()
        return result
    #
    # dontRetry
    # 
    # Don't retry the transfer
    #
    def dontRetry(self):
        result = glite.fts.RetryResult.Hold
        if(self.config.HoldEnabled == False):
            result = glite.fts.RetryResult.Fail
        return result
    #
    # applyForTransferError
    #
    # Last Error is a transfer Error
    #
    def applyForTransferError(self, reason, reasonClass, job, file, transfers):
        result = glite.fts.RetryResult.Wait

        # Srmcopy Error are classified as transfer Errors: detect the File Exists problem

        # Create Patterns
        exists_patterns = PatternList(['.*srm_duplication_error.*',
                                       '.*file.*exists.*cannot write.*',
                                       '.*file.*exists.*'])
        # Check the reason
        if(reason in exists_patterns):
            self.debug("Error Category is Transfer, but from the desciption is a "\
                   "<File exists> failure => Consider it as DestinationError")
            return self.applyForSrmDestError(reason,reasonClass,job,file,transfers)
        # Transfer Error
        self.debug("Transfer Error for File [%s] (reason: <%s>) => Do the default" \
                   % (file.id,reason) )
        delay = self.config.DefaultRetryDelay
        # Check Elapsed Time
        result = self.checkElapsedTime(job,file,transfers,delay)
        if(result == glite.fts.RetryResult.Retry):
            self.info("A transfer error (probably a transient one) happened on "\
                "File [%s]: this file is going to be retried since more than %d seconds elapsed" \
                % (file.id,delay))
        return result
    #
    # applyForSrmSourceError
    #
    # Last Error is an Srm Source Error
    #
    def applyForSrmSourceError(self, reason, reasonClass, job, file, transfers):
        result = glite.fts.RetryResult.Wait
        # Create Patterns
        timeout_patterns = PatternList(['.*get.*timed out.*',
                                        '.*get.*timeout.*'])
        no_file_patterns = PatternList(['.*srm_invalid_path.*',
                                        '.*get.*does not exist.*',
                                        '.*get.*doesn\'t exist.*',
                                        '.*get.*don\'t exist.*',
                                        '.*no such file.*'])
        # Check the reason 
        if(reason in timeout_patterns):
            # timeout on get: probably the file is going to be staged
            self.debug("File [%s] timeout on get (reason: <%s>). Probably is going to be staged" \
                      % (file.id,reason) )
            # Check if previos transfers fails for the same reson
            if((len(transfers) > 1) and (transfers[1].reason in timeout_patterns)):
                # Double the delay time
                self.debug("File [%s] already failed for timeout on get. Double the default delay" \
                         % (file.id) )
                delay = 2 * self.config.RetryDelayForTimeoutOnGet
            else :
                delay = self.config.RetryDelayForTimeoutOnGet
            # Check Elapsed Time
            result = self.checkElapsedTime(job,file,transfers,delay)
            if(result == glite.fts.RetryResult.Retry):
                self.info("File [%s] timed out on get and is going to be retried "\
                          "since more than %d seconds elapsed" % (file.id,delay))
        elif(reason in no_file_patterns):
            # file doesn't exist: no need to retry
            self.info("Source for File [%s] doesn't exist (reason: <%s>) => Fail" \
                     % (file.id,reason) )
            result = glite.fts.RetryResult.Fail
        else: 
            # Unrecognized condition: do the default
            self.debug("Generic condition on File [%s] for Source Error (reason: <%s>) "\
                       "=> Do the default" % (file.id,reason) )
            delay = self.config.DefaultRetryDelay
            # Check if enough time is elapsed
            result = self.checkElapsedTime(job,file,transfers,delay)
            if(result == glite.fts.RetryResult.Retry):
                self.info("A probably transient error happened on File [%s]: "\
                    "this file is going to be retried since more than %d seconds elapsed" \
                    % (file.id,delay))
        # Return Retry Action
        return result
        
    #
    # applyForSrmDestError
    #
    # Last Error is an Srm Destination Error
    #
    def applyForSrmDestError(self, reason, reasonClass, job, file, transfers):
        result = glite.fts.RetryResult.Wait
        # Create Patterns
        exists_patterns = PatternList(['.*error.*during.*transfer.*phase.*',
                                       '.*error.*during.*finalization.*phase.*',
                                       '.*srm_duplication_error.*',
                                       '.*file.*exists.*cannot write.*',
                                       '.*file.*exists.*',
                                       '.*srm_file_busy.*',
                                       '.*device.*busy.*'])
        # SRM request token of last transfer error
        last_tx_error_srm_token = ''
        # Check the reason 
        if(reason in exists_patterns):
            # Check the number of consecutive "file exists" errors 
            count = 1;
            for t in transfers[1:]:
                if(t.reason in exists_patterns):
                    count = count + 1
                else :
                    last_tx_error_srm_token = t.destSrmToken
                    break;
            # check the number of consecutives "file exists" erros
            if(count >= self.config.MaxFileExistsFailures):
                self.warn("File [%s] has failed %d time(s) with a already exists "\
                          "error(last reason: %s) => Don't Retry" % (file.id,count,reason))
                return self.dontRetry()
            # file exists : probably the file is going to be staged
            self.debug("File [%s] already exists (reason: %s)" % (file.id,reason) )
            delay = self.config.RetryDelayForDestFileExists
            # Check Elapsed Time
            result = self.checkElapsedTime(job,file,transfers,delay)
            if(result == glite.fts.RetryResult.Retry):
                # Check if the destination should be overwritten
                if(self.overwriteDestination(job,file,transfers) == True):
                    self.info("File [%s] already exists on destination SE. "\
                               "Try to do abort previous srm request and "\
                               "then delete the file" % (file.destSurl))
                    # Try to abort the previous request 
                    self.abortPreviousDestination(job,file,last_tx_error_srm_token);
                    # Try to delete destination file
                    self.deleteDestination(job,file);
                    # Finally, retry the file with the overwrite flag
                    result = glite.fts.RetryResult.RetryWithOverwrite
                self.info("File [%s] did already exist on destination SE and "\
                          "is going to be retried since more than %d seconds elapsed" \
                          % (file.id,delay))
        else: 
            # Unrecognized condition: do the default
            self.debug("Generic condition on File [%s] for Destination Error "\
                       "(reason %s) => Do the default" % (file.id,reason) )
            delay = self.config.DefaultRetryDelay
            # Check Elapsed time
            result = self.checkElapsedTime(job,file,transfers,delay)
            if(result == glite.fts.RetryResult.Retry):
                self.info("A probably transient error happened for File [%s] on "\
                         "the Destination SE: this file is going to be retried "\
                         "since more than %d seconds elapsed" % (file.id,delay))
        # Return Retry Action
        return result
    #
    # applyForSecurityError
    #
    # Last Error is a Security Error
    #
    def applyForSecurityError(self, reason, reasonClass, job, file, transfers):
        result = glite.fts.RetryResult.Wait
        
        # Create Patterns
        transfer_patterns = PatternList(['.*transfer.*error.*during.*transfer.*phase.*'])

        # Check the reason 
        if(reason in transfer_patterns):
            self.debug("Error Category is Security, but from the desciption is a "\
                   "transfer failure => Consider it as TransferError")
            # Consider it a transfer Error
            result = self.applyForTransferError(reason,reasonClass,job,file,transfers)
        else:
            # Security Error: Don't Retry
            self.info("Security Error on File [%s] (reason: <%s>) => Don't Retry" \
                      % (file.id,reason) )
            result = self.dontRetry()
        return result
    #
    # checkElapsedTime
    #
    # Check if the file should be retried based on the Elapsed Time and the 
    # given delay
    #
    def checkElapsedTime(self, job, file, transfers, delay):
        result = glite.fts.RetryResult.Wait
        # Get Current Time
        transfer_time = 0
        current = time.time()
        # Get the time of last failure
        if(-1 != file.finishTime) :
            # Take the file finish time: it contains the last error
            transfer_time = file.finishTime
        else :
            self.debug("No File finish time set. Consider the last transfer start time as reference")
            if(len(transfers) > 0) :
                # Take the last transfer Time
                transfer_time = transfers[0].transferTime;
            else:
                # File is failed with no transfer attempts. Take the Job's
                # Submission Time
                self.debug("Num Failures: %d - Current %d - Job Submission Time: <%s>" \
                   % (file.failures, file.currentFailures, time.asctime(time.gmtime(job.submitTime))))
                if file.failures > 0 :
                    transfer_time = job.submitTime + (file.failures - 1) * self.config.DefaultRetryDelay;
                else:
                    transfer_time = job.submitTime;
        # Check if it's time to resubmit the File
        self.debug("Last Transfer Reference Time: <%s> - CurrentTime: <%s> " \
            % (time.asctime(time.gmtime(transfer_time)), time.asctime(time.gmtime(current))))
        if (current > transfer_time) and ((current - transfer_time) >= delay) :
            # The File  can be retried
            self.debug("Reschedule File [%s]. Number of tries already performed %d" \
                      % (file.id, file.currentFailures))
            result = glite.fts.RetryResult.Retry;
        else:
            # Elapsed time not sufficient
            self.debug("File [%s] still in Waiting" % (file.id))
        # Return Retry Action
        return result
    #
    # overwriteDestination
    #
    # Retrun true if the destination file should be overwritten
    #
    def overwriteDestination(self, job, file,transfers):
        overwrite = self.config.OverwriteExistingFiles;
        if(job.overwriteFlag == True):
            overwrite = True
        elif(overwrite == False and len(transfers) > 1):
            # Check if there is a previous attemp that failed with a transfer 
            # error. Probably there atre nicer ways to do it , but for the 
            # moment, check just some known cases
            transfer_failed = PatternList(['.*fail.*advisorydelete.*',
                                           '.*transfer.*failed.*',
                                           '.*transfer.*timed out.*',
                                           '.*transfer.*timeout.*',
                                           '.*sizes.*don\'t match.*',
                                           '.*sizes.*do not match.*',
                                           '.*cannot retrieve.*final.*message.*',
                                           '.*invalid.*state.*returned.*transfer-url-copy.*'
                                           ])
            transfer_err_exclude = PatternList(['.*srm.*copy.*'])
            # Iterate on the transfers' list in reverse order
            r_transfers = transfers[:] # shallow-copy
            r_transfers.reverse()
            for t in r_transfers :
                self.debug("Checking File Reason(%s:%s)" % (str(t.reasonClass),t.reason))
                if((t.reasonClass  == glite.fts.ErrorCategory.Transfer) or \
                   (t.reasonClass == glite.fts.ErrorCategory.Agent)    or \
                   (t.reasonClass == glite.fts.ErrorCategory.Network) ):
                    if(t.reason in transfer_failed):
                    #and (t.reason not in transfer_err_exclude)):
                        # A Transfer Error happened and probably the destination
                        # file was not cleaned up
                        self.info("File [%s] already failed during a transfer (reason: <%s>)"\
                              " and probably was not cleaned up correctly" \
                              % (file.sourceSurl,t.reason))
                        overwrite = self.config.OverwriteFailedFiles
                        break
                    self.debug("File [%s] already failed with a transfer error <%s>" \
                            % (file.sourceSurl,t.reason))
                    continue
                elif(t.reasonClass == glite.fts.ErrorCategory.Destination):
                    # file already failed with a destination error
                    self.info("File [%s] already failed with a destination error" \
                           "(reason: <%s>)"  % (file.sourceSurl,t.reason))
                    break
                else:
                    self.debug("File [%s] already failed with error(%s:%s)" \
                            % (file.sourceSurl,str(t.reasonClass),t.reason))
                    continue
        return overwrite;
    #
    # deleteDestination
    #
    # Try to delete the file on the destination SE
    #
    def deleteDestination(self, job, file):
        # Get the Destination SURL
        surl = self.escapeString(file.destSurl);
        # parse Url
        surl_obj = glite.data.agents.utils.Url(surl)
        if(surl_obj.isGood == False):
            self.warn("Cannot delete file since the destination SURL is invalid")
            return
        # Get the Destination SRM endpoint
        srm_service = glite.data.agents.sd.GetService(surl_obj.host,self.config.SrmServiceType,job.voName)
        if(srm_service is None):
            self.warn("Cannot delete file since the destination SE doesn't exist")
            return
        # Get Endpoint and version
        endpoint = self.escapeString(srm_service.endpoint);
        srm_version = srm_service.version;
        # Check the operating system
        if re.match("posix",os.name):
            check_cmd_1 = "which glite-srm-util-rm > /dev/null 2>&1"
            del_cmd_1   = "glite-srm-util-rm -s \"%s\" -I \"%s\" \"%s\" > /dev/null 2>&1" \
                           % (endpoint,srm_version,surl)
            check_cmd_2 = "which ${GLITE_LOCATION}/libexec/glite-srm-util-rm" \
                          " > /dev/null 2>&1"
            del_cmd_2   = "${GLITE_LOCATION}/libexec/glite-srm-util-rm " \
                          "-s \"%s\" -I \"%s\" \"%s\" > /dev/null 2>&1" \
                           % (endpoint,srm_version,surl)
            check_cmd_3 = "which glite-srm-delete > /dev/null 2>&1"
            del_cmd_3   = "glite-srm-delete -s \"%s\" \"%s\" > /dev/null 2>&1" \
                           % (endpoint,surl)
        else:
            self.warn("Cannot delete file since the current OS is not supported")
            return
        # check if the deletion command exists
        del_cmd = ""
        self.debug("Executing statement <%s>" % (check_cmd_1))
        result = os.system(check_cmd_1)
        if(result == 0):
            del_cmd = del_cmd_1
        else:
            self.debug("Executing statement <%s>" % (check_cmd_2))
            result = os.system(check_cmd_2)
            if(result == 0):
                del_cmd = del_cmd_2
            else:
                self.debug("Executing statement <%s>" % (check_cmd_3))
                result = os.system(check_cmd_3)
                if(result == 0):
                    del_cmd = del_cmd_3

        # if the deletion command exists, execute it
        if(result == 0):
            self.info("Trying to delete the SURL %s on the destination SE (%s)" \
                     % (surl,endpoint))
            # Execute the statement
            self.debug("Executing statement <%s>" % (del_cmd))
            result = os.system(del_cmd)
            if(result == 0):
                self.info("Command to delete SURL %s on destination SE "\
                          "successfully executed" % (surl))
            else:
                self.warn("Failed to delete SURL %s on destination SE" % (surl))
        else:
            self.info("Cannot delete the SURL %s since the required tool is "\
                      "missing" % (surl))
    #
    # abortPreviousDestination
    #
    # Try to abort the file on the destination SE
    #
    def abortPreviousDestination(self, job, file, srm_token):
        # Get the Destination SURL
        surl = self.escapeString(file.destSurl);
        if(len(srm_token) <= 0):
            self.debug("Cannot abort the SURL %s since the token of the previous srm "\
                      "request is missing" % (surl))
            return
        # parse Url
        surl_obj = glite.data.agents.utils.Url(surl)
        if(surl_obj.isGood == False):
            self.warn("Cannot abort file since the destination SURL is invalid")
            return
        # Get the Destination SRM endpoint
        srm_service = glite.data.agents.sd.GetService(surl_obj.host,self.config.SrmServiceType,job.voName)
        if(srm_service is None):
            self.warn("Cannot abort file since the destination SE doesn't exist")
            return
        # Get Endpoint and version
        endpoint = self.escapeString(srm_service.endpoint);
        srm_version = srm_service.version;
        # Check the operating system
        if re.match("posix",os.name):
            check_cmd_1 = "which glite-srm-util-abortfiles > /dev/null 2>&1"
            abort_cmd_1 = "glite-srm-util-abortfiles -s \"%s\" -I \"%s\" \"%s\" \"%s\" > /dev/null 2>&1" \
                           % (endpoint,srm_version,srm_token,surl)
            check_cmd_2 = "which ${GLITE_LOCATION}/libexec/glite-srm-util-abortfiles" \
                          " > /dev/null 2>&1"
            abort_cmd_2 = "${GLITE_LOCATION}/libexec/glite-srm-util-abortfiles " \
                          "-s \"%s\" -I \"%s\" \"%s\" \"%s\" > /dev/null 2>&1" \
                           % (endpoint,srm_version,srm_token,surl)
        else:
            self.warn("Cannot abort file since the current OS is not supported")
            return
        # check if the deletion command exists
        abort_cmd = ""
        self.debug("Executing statement <%s>" % (check_cmd_1))
        result = os.system(check_cmd_1)
        if(result == 0):
            abort_cmd = abort_cmd_1
        else:
            self.debug("Executing statement <%s>" % (check_cmd_2))
            result = os.system(check_cmd_2)
            if(result == 0):
                abort_cmd = abort_cmd_2

        # if the abort command exists, execute it
        if(result == 0):
            self.info("Trying to abort the SURL %s on the destination SE (%s)" \
                     % (surl,endpoint))
            # Execute the statement
            self.debug("Executing statement <%s>" % (abort_cmd))
            result = os.system(abort_cmd)
            if(result == 0):
                self.info("Command to abort SURL %s on destination SE "\
                          "successfully executed" % (surl))
            else:
                self.warn("Failed to abort SURL %s on destination SE" % (surl))
        else:
            self.info("Cannot abort the SURL %s since the required tool is "\
                      "missing" % (surl))
    #
    # escapeString
    #
    # Escape as string to avoid URL injection
    #
    def escapeString(self, str):
        escape_str = str.replace("\\", "\\\\")
        escape_str = escape_str.replace("\"", "\\\"")
        escape_str = escape_str.replace(";", "\\;")
        return escape_str;

################################################################################
#
# Public Methods
#
################################################################################
#
# The Retry Config Parameter
#
g_RetryConfig  = None
#
# Declare the Retry Version supported by this script
#
def RetryVersion():
    return "1.0"
#
# Declare the Retry Version supported by this script
#
def InitRetry(params = ""):
    # Create Config Object
    global g_RetryConfig
    if(g_RetryConfig is None):
        g_RetryConfig = SmarterRetryConfiguration(params)
    return True
#
# Apply the Retry Strategy to the given File
#
def Retry(job,file,transfers):
    # Create Strategy Object
    strategy = SmarterRetryStrategy(g_RetryConfig)
    # Apply logic
    return strategy.apply(job,file,transfers)
